package fi.cas.formplugin;

import com.alibaba.fastjson.JSON;
import kd.bos.bill.BillOperationStatus;
import kd.bos.bill.BillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.resource.ResManager;
import kd.bos.dataentity.serialization.SerializationUtils;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.AppMetadataCache;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.param.AppParam;
import kd.bos.exception.KDBizException;
import kd.bos.ext.tmc.utils.commitToBe.enums.TmcAppEnum;
import kd.bos.filter.FilterColumn;
import kd.bos.form.*;
import kd.bos.form.control.events.BeforeItemClickEvent;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.*;
import kd.bos.form.operate.AbstractOperate;
import kd.bos.list.BillList;
import kd.bos.list.IListView;
import kd.bos.list.ListShowParameter;
import kd.bos.list.events.BeforeShowBillFormEvent;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.mvc.list.ListView;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.MetadataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.servicehelper.parameter.SystemParamServiceHelper;
import kd.fi.cas.business.errorcode.ReceivErrorCode;
import kd.fi.cas.business.helper.ManualMatchHelper;
import kd.fi.cas.business.helper.RecDraftHelper;
import kd.fi.cas.business.paywarning.RecWarningHelper;
import kd.fi.cas.enums.*;
import kd.fi.cas.formplugin.helper.InitCheckPluginHelper;
import kd.fi.cas.formplugin.refund.log.RefundLogExecutor;
import kd.fi.cas.helper.*;
import kd.fi.cas.util.EmptyUtil;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

public class RecBillListPlugin extends AbstractListPlugin {
    private static final String KEY_CALLBACK_RECCHARGEBACK = "CALLBACK_RECCHARGEBACK";
    private static final String CONFIRM_DEL_CALLBACK = "CONFIRM_DEL_CALLBACK";
    private static final String CONFIRM_DELETE_CALLBACK = "CONFIRM_DELETE_CALLBACK";
    private static final String KEY_CLOSECALLBACK_CHARGEBACK = "CALLBACK_SELECTDRAFT";
    private FilterContainerInitArgs initArgs;
    private static final String BAR_BATCH_INPUT = "batchinput";
    private RefundLogExecutor refundLogExecutor;

    public RecBillListPlugin() {
    }

    public void filterContainerInit(FilterContainerInitArgs args) {
        super.filterContainerInit(args);
        this.initArgs = args;
        ListShowParameter listShowParameter = (ListShowParameter)this.getView().getFormShowParameter();
        Object isFromDraftBill = listShowParameter.getCustomParams().getOrDefault("isFromDraftBill", (Object)null);
        Iterator var4;
        FilterColumn filterColumn;
        if (null == this.getCachedOrgList()) {
            var4 = args.getCommonFilterColumns().iterator();

            while(var4.hasNext()) {
                filterColumn = (FilterColumn)var4.next();
                if ("org.id".equals(filterColumn.getFieldName())) {
                    List<Object> defaultValues = filterColumn.getDefaultValues();
                    if (null != defaultValues && defaultValues.size() > 0) {
                        List<Long> selectedOrgIdList = (List)defaultValues.stream().filter((t) -> {
                            return StringUtils.isNotEmpty(t.toString());
                        }).map((item) -> {
                            return Long.valueOf(item.toString());
                        }).collect(Collectors.toList());
                        this.cacheSelectedOrgIdList(selectedOrgIdList);
                    }
                }
            }
        }

        if (CasPluginHelper.isTrack(this.getView().getFormShowParameter())) {
            var4 = args.getCommonFilterColumns().iterator();

            while(var4.hasNext()) {
                filterColumn = (FilterColumn)var4.next();
                if ("org.id".equals(filterColumn.getFieldName())) {
                    filterColumn.setDefaultValue("");
                } else if ("billstatus".equals(filterColumn.getFieldName())) {
                    filterColumn.setDefaultValues(new Object[]{""});
                } else if ("bizdate".equals(filterColumn.getFieldName())) {
                    filterColumn.setDefaultValue("");
                }
            }
        }

        if (EmptyUtil.isNoEmpty(isFromDraftBill) && isFromDraftBill.toString().trim().equals("Y")) {
            CasPluginHelper.getCasQFilter(listShowParameter, args);
        }

        this.fillToolBar();
    }

    public void filterContainerSearchClick(FilterContainerSearchClickArgs args) {
        super.filterContainerSearchClick(args);
        List<Long> selectedOrgIdList = args.getSelectMainOrgIds();
        if (selectedOrgIdList.size() > 0) {
            List<Object> cachedOrgList = this.getCachedOrgList();
            if (null == cachedOrgList || !cachedOrgList.toString().equals(selectedOrgIdList.toString())) {
                this.cacheSelectedOrgIdList(selectedOrgIdList);
                this.filterContainerInit(this.initArgs);
            }
        }

    }

    private void cacheSelectedOrgIdList(List<Long> selectedOrgIdList) {
        if (selectedOrgIdList != null) {
            this.getPageCache().put("selectedOrgIdList", SerializationUtils.toJsonString(selectedOrgIdList));
        } else {
            this.getPageCache().put("selectedOrgIdList", (String)null);
        }

    }

    protected List<Object> getCachedOrgList() {
        return this.getPageCache().get("selectedOrgIdList") != null ? (List)SerializationUtils.fromJsonString(this.getPageCache().get("selectedOrgIdList"), List.class) : null;
    }

    private void fillToolBar() {
        this.getView().setVisible(false, new String[]{"recbook"});
        List<Object> tempList = this.getCachedOrgList();
        if (tempList != null && tempList.size() != 0) {
            List<Long> cachedOrgList = (List)this.getCachedOrgList().stream().map((item) -> {
                return Long.valueOf(item.toString());
            }).collect(Collectors.toList());
            Iterator var3 = cachedOrgList.iterator();

            boolean isdraft;
            boolean ischeck;
            boolean ispromissorynote;
            do {
                if (!var3.hasNext()) {
                    return;
                }

                Long orgId = (Long)var3.next();
                isdraft = SystemParameterHelper.getCdmParameterBoolean(orgId, "isdraft");
                ischeck = SystemParameterHelper.getCdmParameterBoolean(orgId, "ischeck");
                ispromissorynote = SystemParameterHelper.getCdmParameterBoolean(orgId, "ispromissorynote");
            } while(!isdraft && !ischeck && !ispromissorynote);

            this.getView().setVisible(true, new String[]{"recbook"});
        } else {
            this.getView().setVisible(true, new String[]{"recbook"});
        }
    }

    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String key = evt.getItemKey();
        switch (key.toLowerCase(Locale.ENGLISH)) {
            case "salesrec":
                this.addNew(RecTypeEnum.SalesRec);
                break;
            case "otherrec":
                this.addNew(RecTypeEnum.OtherRec);
                break;
            case "transcompany":
                this.transCompany(evt);
                break;
            case "tbloffset":
                this.offset();
                break;
            case "viewarsettle":
                CasPluginHelper.viewSettleForRec((BillList)this.getControl("billlistap"), this.getView());
                break;
            case "cancelrec":
                this.cancelrec();
                break;
            case "tblsubmit":
                this.beforeSubmit();
                break;
            case "viewdrafbill":
                CasPluginHelper.viewDrafBill((BillList)this.getControl("billlistap"), this.getView(), "nckd_cas_recbill_xz");
                break;
            case "batchinput":
                CasPluginHelper.viewBatchInput(this.getView(), "cas_batchrecbill");
        }

    }

    private void beforeSubmit() {
        BillList billList = (BillList)this.getControl("billlistap");
        ListSelectedRowCollection selectedRows = billList.getSelectedRows();
        if (selectedRows.size() > 0) {
            List<Object> rowIds = new ArrayList(selectedRows.size());
            selectedRows.forEach((row) -> {
                rowIds.add(row.getPrimaryKeyValue());
            });
            List<DynamicObject> resultList = RecWarningHelper.getResultByPkIds(rowIds);
            if (resultList != null && resultList.size() > 0) {
                FormShowParameter parameter = new FormShowParameter();
                parameter.getOpenStyle().setShowType(ShowType.Modal);
                parameter.setFormId("cas_recwarnmsg");
                parameter.setStatus(OperationStatus.VIEW);
                parameter.setCustomParam("sumAll", selectedRows.size());
                parameter.setCustomParam("sumWarn", resultList.size());
                parameter.setCloseCallBack(new CloseCallBack(this, "beforeSubmit"));
                this.getView().showForm(parameter);
            } else {
                this.getView().invokeOperation("submit");
            }
        }

    }

    private void suppleContractOrOrgsDivide(String opName) {
        BillList billList = (BillList)this.getView().getControl("billlistap");
        ListSelectedRowCollection selectedRows = billList.getSelectedRows();
        if (CasHelper.getSelectedRowSize(selectedRows) != 1) {
            this.getView().showErrorNotification(ResManager.loadKDString("请选中一行再进行操作。", "ReceivingBillList_25", "fi-cas-formplugin", new Object[0]));
        } else {
            ListSelectedRow currentSelectedRowInfo = selectedRows.get(0);
            String billStatus = currentSelectedRowInfo.getBillStatus();
            if (!"D".equals(billStatus)) {
                if ("supplecontract".equals(opName)) {
                    this.getView().showErrorNotification(ResManager.loadKDString("单据状态不符合，不能进行此操作。", "ReceivingBillList_26", "fi-cas-formplugin", new Object[0]));
                } else if ("orgsdivide".equals(opName)) {
                    this.getView().showErrorNotification(ResManager.loadKDString("只有单据状态为已收款，才能操作组织间清分。", "ReceivingBillList_35", "fi-cas-formplugin", new Object[0]));
                }

            } else {
                Object primaryKeyValue = currentSelectedRowInfo.getPrimaryKeyValue();
                DynamicObject recbillDO = BusinessDataServiceHelper.loadSingle(primaryKeyValue, "nckd_cas_recbill_xz", "receivingtype,settletype,actrecamt");
                DynamicObject receivingtype = recbillDO.getDynamicObject("receivingtype");
                boolean ispartreceivable = false;
                if (CasHelper.isNotEmpty(receivingtype)) {
                    ispartreceivable = receivingtype.getBoolean("ispartreceivable");
                }

                if (!ispartreceivable) {
                    this.getView().showErrorNotification(ResManager.loadKDString("此单据收款类型没有参与应收结算，不能进行此操作。", "ReceivingBillList_27", "fi-cas-formplugin", new Object[0]));
                } else if (BigDecimal.ZERO.compareTo(recbillDO.getBigDecimal("actrecamt")) > 0) {
                    this.getView().showErrorNotification(ResManager.loadKDString("此单据收款金额为负数，不能进行此操作。", "ReceivingBillList_32", "fi-cas-formplugin", new Object[0]));
                } else {
                    BillShowParameter showParameter = new BillShowParameter();
                    showParameter.setCustomParam("supcontractOrOrgsdivide", opName);
                    showParameter.setPkId(primaryKeyValue);
                    showParameter.setFormId("cas_recbill_supdevide");
                    showParameter.setCaption(ResManager.loadKDString("收款单", "ReceivingBillList_28", "fi-cas-formplugin", new Object[0]));
                    showParameter.getOpenStyle().setShowType(ShowType.MainNewTabPage);
                    this.getView().showForm(showParameter);
                }
            }
        }
    }

    public void closedCallBack(ClosedCallBackEvent e) {
        super.closedCallBack(e);
        if ("listoperation".equals(e.getActionId())) {
            this.getView().invokeOperation("refresh");
        } else if ("refund".equals(e.getActionId())) {
            ((ListView)this.getView()).refresh();
        } else {
            OperateOption option;
            if (StringUtils.equals(e.getActionId(), "beforeSubmit")) {
                Object returnData = e.getReturnData();
                if (returnData != null) {
                    if ((Boolean)returnData) {
                        this.getView().invokeOperation("submit");
                    } else {
                        option = OperateOption.create();
                        option.setVariableValue("isTrue", String.valueOf(returnData));
                        this.getView().invokeOperation("warnsubmit", option);
                        this.getView().invokeOperation("refresh");
                    }
                } else {
                    this.getView().invokeOperation("warnsubmit");
                    this.getView().invokeOperation("refresh");
                }
            } else if (StringUtils.equals(e.getActionId(), "CALLBACK_RECCHARGEBACK")) {
                Map<String, Object> data = (Map)e.getReturnData();
                if (null != data) {
                    option = OperateOption.create();
                    option.setVariableValue("billid", String.valueOf(data.get("billid")));
                    option.setVariableValue("reason", String.valueOf(data.get("reason")));
                    option.setVariableValue("entityname", "nckd_cas_recbill_xz");
                    OperationResult recchargebackwrite = this.getView().invokeOperation("recchargebackwrite", option);
                    if (recchargebackwrite.isSuccess()) {
                        this.getView().showSuccessNotification(ResManager.loadKDString("退单成功。", "ReceivingBillList_29", "fi-cas-formplugin", new Object[0]));
                        this.getView().invokeOperation("refresh");
                    }
                }
            } else if (StringUtils.equals(e.getActionId(), "CALLBACK_SELECTDRAFT")) {
                Map<String, Object> returnData = (HashMap)e.getReturnData();
                if (null != returnData) {
                    Object draftIdList = returnData.get("draftBillIdList");
                    Object recBillId = returnData.get("recBillId");
                    if (draftIdList != null) {
                        this.displayDraftInfo(draftIdList, recBillId);
                        option = OperateOption.create();
                        option.setVariableValue("draftids", JSON.toJSONString(draftIdList));
                        option.setVariableValue("selectdraft", "true");
                        this.getView().invokeOperation("receivingrec", option);
                    }
                }
            }
        }

    }

    private void displayDraftInfo(Object draftIdList, Object recBillId) {
        if (!EmptyUtil.isEmpty(draftIdList) || !EmptyUtil.isEmpty(recBillId)) {
            DynamicObject recBill = BusinessDataServiceHelper.loadSingle((Long)recBillId, "nckd_cas_recbill_xz");
            if (!EmptyUtil.isEmpty(recBill)) {
                DynamicObjectCollection draftInfoEntrys = recBill.getDynamicObjectCollection("cas_draftinfo");
                List<Object> draftIds = (List)draftIdList;
                DynamicObject[] draftBills = BusinessDataServiceHelper.load(draftIds.toArray(), EntityMetadataCache.getDataEntityType("cdm_draftbillf7"));

                for(int i = 0; i < draftBills.length; ++i) {
                    DynamicObject newEntry = draftInfoEntrys.addNew();
                    newEntry.set("draftbillinfo", draftBills[i].getLong("id"));
                    newEntry.set("recbillcurrency", draftBills[i].getDynamicObject("currency").getLong("id"));
                    newEntry.set("transamount", draftBills[i].getBigDecimal("availableamount"));
                    newEntry.set("draftbilllogid", 0);
                    newEntry.set("srcbilltype", "nckd_cas_recbill_xz");
                    newEntry.set("srcbillindex", 1);
                }

                SaveServiceHelper.save(new DynamicObject[]{recBill});
            }

        }
    }

    private void offset() {
        BillList list = (BillList)this.getView().getControl("billlistap");
        ListSelectedRowCollection selectedRows = list.getSelectedRows();
        if (selectedRows != null) {
            Object[] pks = selectedRows.getPrimaryKeyValues();
            if (pks.length <= 0) {
                this.getView().showErrorNotification(ResManager.loadKDString("没有选择任何数据，请先选择数据。", "ReceivingBillList_4", "fi-cas-formplugin", new Object[0]));
            } else if (pks.length > 1) {
                this.getView().showErrorNotification(ResManager.loadKDString("不支持批量操作，请重新选择数据。", "ReceivingBillList_5", "fi-cas-formplugin", new Object[0]));
            } else {
                this.getPageCache().put("operate", OperateTypeEnum.OFFSET.name());
                this.getView().invokeOperation("copy");
                this.getPageCache().remove("operate");
            }
        } else {
            this.getView().showErrorNotification(ResManager.loadKDString("没有选择任何数据，请先选择数据。", "ReceivingBillList_4", "fi-cas-formplugin", new Object[0]));
        }

    }

    private void transCompany(ItemClickEvent evt) {
        BillList billList = (BillList)this.getControl("billlistap");
        ListSelectedRowCollection selectedRows = billList.getSelectedRows();
        if (CasHelper.getSelectedRowSize(selectedRows) != 1) {
            this.getView().showErrorNotification(ResManager.loadKDString("请选择一条记录进行指定实际收款公司操作。", "ReceivingBillList_6", "fi-cas-formplugin", new Object[0]));
        } else {
            String appId = this.getView().getFormShowParameter().getAppId();
            if (EmptyUtil.isNotEmpty(appId)) {
                appId = AppMetadataCache.getAppInfo(appId).getId();
            } else {
                appId = AppMetadataCache.getAppInfo("cas").getId();
            }

            List<Long> orgList = OrgHelper.getAuthorizedBankOrgId(Long.valueOf(RequestContext.get().getUserId()), appId, "nckd_cas_recbill_xz", "4715a0df000000ac");
            if (orgList != null && orgList.size() != 0) {
                IListView listView = (IListView)this.getView();
                List<Long> selectedMainOrgIds = listView.getSelectedMainOrgIds();
                if (selectedMainOrgIds != null && selectedMainOrgIds.size() > 0 && !orgList.contains(selectedMainOrgIds.get(0))) {
                    this.getView().showErrorNotification(ResManager.loadKDString("您没有该组织的指定实收公司权限。", "ReceivingBillList_7", "fi-cas-formplugin", new Object[0]));
                } else {
                    ListSelectedRow selectedRow = selectedRows.get(0);
                    Long receivingId = (Long)selectedRow.getPrimaryKeyValue();
                    if (this.check(receivingId, evt)) {
                        FormShowParameter showParameter = new FormShowParameter();
                        showParameter.setFormId("cas_rec_transcompany");
                        showParameter.setCustomParam("receId", receivingId);
                        showParameter.getOpenStyle().setShowType(ShowType.NonModal);
                        this.getView().showForm(showParameter);
                    }

                }
            } else {
                this.getView().showErrorNotification(ResManager.loadKDString("您没有该组织的指定实收公司权限。", "ReceivingBillList_7", "fi-cas-formplugin", new Object[0]));
            }
        }
    }

    private boolean check(Long receivingId, ItemClickEvent evt) {
        DynamicObject receivable = BusinessDataServiceHelper.loadSingle(receivingId, "nckd_cas_recbill_xz", "billno,billstatus,isagent");
        if ("D".equals(receivable.getString("billstatus"))) {
            this.getView().showErrorNotification(receivable.getString("billno") + (new ReceivErrorCode()).STATUS_TRANS_COMPANY().getMessage());
            return false;
        } else if (!receivable.getBoolean("isagent")) {
            this.getView().showErrorNotification(receivable.getString("billno") + (new ReceivErrorCode()).STATUS_RECE_TYPE().getMessage());
            return false;
        } else {
            return true;
        }
    }

    private void addNew(RecTypeEnum typeEnum) {
        this.getPageCache().put("recmentType", typeEnum.name());
        this.getView().invokeOperation("new");
    }

    public void beforeShowBill(BeforeShowBillFormEvent e) {
        BillShowParameter parameter = e.getParameter();
        FormShowParameter formShowParameter = new FormShowParameter();
        BillOperationStatus billStatus = parameter.getBillStatus();
        Long billId = (Long)parameter.getPkId();
        if (billId != null) {
            DynamicObject info = BusinessDataServiceHelper.loadSingle(billId, "nckd_cas_recbill_xz", "id, billstatus,biztype");
            if (parameter.getStatus() == OperationStatus.ADDNEW && billId != null) {
                String operateType = this.getPageCache().get("operate");
                BillShowParameter showParameter = e.getParameter();
                if (operateType != null) {
                    showParameter.getCustomParams().put("operate", operateType);
                } else {
                    showParameter.getCustomParams().put("operate", OperateTypeEnum.COPY);
                }
            }

            if (info.getString("biztype").equals(RecTypeEnum.OtherRec.toString())) {
                formShowParameter.setCaption(ResManager.loadKDString("其他收款单", "ReceivingBillList_8", "fi-cas-formplugin", new Object[0]));
                formShowParameter.getCustomParams().put("recmentType", RecTypeEnum.OtherRec);
                parameter.setCaption(ResManager.loadKDString("其他收款单", "ReceivingBillList_8", "fi-cas-formplugin", new Object[0]));
                parameter.getCustomParams().put("recmentType", RecTypeEnum.OtherRec);
            } else {
                formShowParameter.setCaption(ResManager.loadKDString("销售收款单", "ReceivingBillList_9", "fi-cas-formplugin", new Object[0]));
                formShowParameter.getCustomParams().put("recmentType", RecTypeEnum.SalesRec);
                parameter.setCaption(ResManager.loadKDString("销售收款单", "ReceivingBillList_9", "fi-cas-formplugin", new Object[0]));
                parameter.getCustomParams().put("recmentType", RecTypeEnum.SalesRec);
            }
        } else if (BillOperationStatus.ADDNEW == billStatus) {
            parameter.getCustomParams().put("recmentType", this.getPageCache().get("recmentType"));
            this.getView().showForm(parameter);
        }

        Map<String, Object> customParam = new HashMap();
        customParam.put("EntityName", parameter.getFormId());
        if (!OperationStatus.ADDNEW.equals(e.getParameter().getStatus())) {
            customParam.put("PkId", parameter.getPkId());
            formShowParameter.setCustomParams(customParam);
        } else {
            this.getView().showForm(parameter);
        }
    }

    public void beforeDoOperation(BeforeDoOperationEventArgs args) {
        super.beforeDoOperation(args);
        AbstractOperate operate = (AbstractOperate)args.getSource();
        String opKey = operate.getOperateKey();
        BillList billList = (BillList)this.getControl("billlistap");
        ListSelectedRowCollection selectedRows = billList.getSelectedRows();
        Long id;
        if ("cancelrefund".equals(opKey)) {
            if (CasHelper.getSelectedRowSize(selectedRows) > 1) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("请选择一笔收款单进行取消退款退票。", "ReceivingBillList_10_1", "fi-cas-formplugin", new Object[0]));
            }

            id = (Long)selectedRows.get(0).getPrimaryKeyValue();
            operate.getOption().setVariableValue("revId", String.valueOf(id));
            this.refundLogExecutor = new RefundLogExecutor(id);
        }

        boolean isSettleCenterCollect;
        Set srcBill;
        if ("refund".equals(opKey)) {
            if (selectedRows.size() == 0) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("请选择要执行的数据。", "ReceivingBillList_0", "fi-cas-formplugin", new Object[0]));
            }

            if (CasHelper.getSelectedRowSize(selectedRows) > 1) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("请选择单条数据。", "ReceivingBillList_10", "fi-cas-formplugin", new Object[0]));
            }

            if (!RefundHelper.isPaid((Long)selectedRows.get(0).getPrimaryKeyValue())) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("只有已收款状态的单据才可以进行退票操作。", "ReceivingBillList_11", "fi-cas-formplugin", new Object[0]));
            }

            if (RefundHelper.isRefund((Long)selectedRows.get(0).getPrimaryKeyValue())) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("已确认退票的收款单不允许重复退票。", "ReceivingBillList_12", "fi-cas-formplugin", new Object[0]));
            }

            isSettleCenterCollect = QueryServiceHelper.exists("nckd_cas_recbill_xz", new QFilter[]{new QFilter("id", "=", selectedRows.get(0).getPrimaryKeyValue()), new QFilter("receivingtype.biztype", "=", PayAndRecBizTypeEnum.SETTLCENTER_COLLECT.getValue())});
            if ((!isSettleCenterCollect || !RefundHelper.isExistTransHandleBillsRefund((Long)selectedRows.get(0).getPrimaryKeyValue())) && (isSettleCenterCollect || !RefundHelper.isExistPayBills((Long)selectedRows.get(0).getPrimaryKeyValue()))) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("您选择的收款单找不到对应的退款源单，确认退票失败。", "ReceivingBillList_13", "fi-cas-formplugin", new Object[0]));
            }

            srcBill = CasBotpHelper.getSrcBillByDestBill((Long)selectedRows.get(0).getPrimaryKeyValue(), "nckd_cas_recbill_xz", "cas_paybill");
            if (!CasHelper.isEmpty(srcBill)) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("您选择的收款单已关联付款单，不允许退票。", "ReceivingBillList_17", "fi-cas-formplugin", new Object[0]));
            }

            if (RefundHelper.isSettled((Long)selectedRows.get(0).getPrimaryKeyValue())) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("已结算的收款单不允许退票，请先手工取消结算。", "ReceivingBillList_33", "fi-cas-formplugin", new Object[0]));
            }
        } else if ("returnmoney".equals(opKey)) {
            isSettleCenterCollect = QueryServiceHelper.exists("nckd_cas_recbill_xz", new QFilter[]{new QFilter("id", "=", selectedRows.get(0).getPrimaryKeyValue()), new QFilter("receivingtype.biztype", "=", PayAndRecBizTypeEnum.SETTLCENTER_COLLECT.getValue())});
            if (selectedRows.size() == 0) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("请选择要执行的数据。", "ReceivingBillList_0", "fi-cas-formplugin", new Object[0]));
            }

            if (CasHelper.getSelectedRowSize(selectedRows) > 1) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("请选择单条数据。", "ReceivingBillList_10", "fi-cas-formplugin", new Object[0]));
            }

            if (!RefundHelper.isPaid((Long)selectedRows.get(0).getPrimaryKeyValue())) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("只有已收款状态的单据才可以进行退款操作。", "ReceivingBillList_14", "fi-cas-formplugin", new Object[0]));
            }

            if (RefundHelper.isRefund((Long)selectedRows.get(0).getPrimaryKeyValue())) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("已退款的收款单不允许重复退款。", "ReceivingBillList_15", "fi-cas-formplugin", new Object[0]));
            }

            if ((!isSettleCenterCollect || !RefundHelper.isExistTransHandleBillsRefundMoney((Long)selectedRows.get(0).getPrimaryKeyValue())) && (isSettleCenterCollect || !RefundHelper.isExistPayBillsRefund((Long)selectedRows.get(0).getPrimaryKeyValue()))) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("您选择的收款单找不到对应的退款源单，退款失败。", "ReceivingBillList_16", "fi-cas-formplugin", new Object[0]));
            }

            srcBill = CasBotpHelper.getSrcBillByDestBill((Long)selectedRows.get(0).getPrimaryKeyValue(), "nckd_cas_recbill_xz", "cas_paybill");
            if (!CasHelper.isEmpty(srcBill)) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("您选择的收款单已关联付款单，不允许退款。", "ReceivingBillList_18_1", "fi-cas-formplugin", new Object[0]));
            }

            if (RefundHelper.isSettled((Long)selectedRows.get(0).getPrimaryKeyValue())) {
                args.setCancel(true);
                throw new KDBizException(ResManager.loadKDString("已结算的收款单不允许退款，请先手工取消结算。", "ReceivingBillList_34", "fi-cas-formplugin", new Object[0]));
            }
        } else {
            DynamicObject recBill;
            if ("recbook".equals(opKey)) {
                id = (Long)selectedRows.get(0).getPrimaryKeyValue();
                DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle(id, "nckd_cas_recbill_xz");
                DynamicObjectCollection draftBillCol = (DynamicObjectCollection)dynamicObject.get("draftbill");
                if (CasHelper.getSelectedRowSize(selectedRows) > 1) {
                    args.setCancel(true);
                    throw new KDBizException(ResManager.loadKDString("请选择单条数据。", "ReceivingBillList_10", "fi-cas-formplugin", new Object[0]));
                }

                if (!this.checkStatus(id, "billstatus")) {
                    args.setCancel(true);
                    this.getView().showTipNotification(ResManager.loadKDString("只有已审核状态的单据允许收票登记。", "PaymentBillDraftEditPlugin_12", "fi-cas-formplugin", new Object[0]));
                    return;
                }

                if (!InitCheckPluginHelper.checkInitForPlugin(this.getView(), dynamicObject.getDynamicObject("org"), new String[0])) {
                    args.setCancel(true);
                    return;
                }

                if (!DraftHelper.isDraftBill(id, "nckd_cas_recbill_xz")) {
                    args.setCancel(true);
                    this.getView().showTipNotification(ResManager.loadKDString("收票登记仅适用结算方式类型是承兑汇票、支票或本票的收款单，你所选单据不支持收票登记。", "ReceivingBillEdit_12", "fi-cas-formplugin", new Object[0]));
                } else if (!DraftHelper.isParameterDraftOn(id, "nckd_cas_recbill_xz")) {
                    args.setCancel(true);
                    this.getView().showTipNotification(ResManager.loadKDString("该组织没有启用相应的系统参数，请配置相应的系统参数。", "PaymentBillDraftEditPlugin_4", "fi-cas-formplugin", new Object[0]));
                } else if (EmptyUtil.isNoEmpty(draftBillCol)) {
                    args.setCancel(true);
                    this.getView().showTipNotification(ResManager.loadKDString("收款单已选择相关应收票据，不能再次收票登记，请确认收款", "ReceivingBillDraftEditPlugin_1", "fi-cas-formplugin", new Object[0]));
                } else {
                    Boolean isPush = CasBotpHelper.isFindTargetBill("nckd_cas_recbill_xz", "cdm_receivablebill", new Long[]{id});
                    if (isPush) {
                        args.setCancel(true);
                        this.getView().showTipNotification(ResManager.loadKDString("收款单已进行收票登记。", "ReceivingBillEdit_13", "fi-cas-formplugin", new Object[0]));
                    }

                    recBill = BusinessDataServiceHelper.loadSingle("nckd_cas_recbill_xz", "accountbank", new QFilter[]{new QFilter("id", "=", id)});
                    DynamicObject accountBank = recBill.getDynamicObject("accountbank");
                    if (null != accountBank && AccountBankHelper.isClosed(accountBank.getLong("id"))) {
                        throw new KDBizException(String.format(ResManager.loadKDString("账户%s当前状态为已销户，暂无法进行业务处理。", "PaymentErrorCode_47", "fi-cas-business", new Object[0]), accountBank.getString("number")));
                    }
                }
            } else if ("chargeback".equals(opKey)) {
                if (CasHelper.getSelectedRowSize(selectedRows) > 1) {
                    this.getView().showTipNotification(ResManager.loadKDString("退单仅支持对单张单据处理，请修改选择范围。", "PaymentBillList_7", "fi-cas-formplugin", new Object[0]));
                    args.cancel = true;
                }
            } else if ("matchdetail".equals(opKey)) {
                ManualMatchHelper.verifySameOrg(selectedRows, "nckd_cas_recbill_xz", this.getView(), args);
            } else if ("receivingrec".equals(opKey)) {
                BillList list = (BillList)this.getView().getControl("billlistap");
                Object[] selectedRowPks = list.getSelectedRows().getPrimaryKeyValues();
                RecDraftHelper.fillDraft(selectedRowPks, true);
                if (selectedRowPks.length > 0) {
                    DynamicObject[] dbDatas = BusinessDataServiceHelper.load(selectedRowPks, EntityMetadataCache.getDataEntityType("nckd_cas_recbill_xz"));
                    String draftIds = operate.getOption().getVariableValue("draftids", "");
                    if (selectedRowPks.length == 1 && StringUtils.isEmpty(draftIds)) {
                        recBill = dbDatas[0];
                        String sourceBillType = (String)recBill.get("sourcebilltype");
                        if ("cas_claimcenterbill".equals(sourceBillType)) {
                            return;
                        }

                        DynamicObject orgData = (DynamicObject)recBill.get("org");
                        Long pkValue = (Long)orgData.getPkValue();
                        boolean selectDraftFlag = SystemParameterHelper.getParameterBoolean(pkValue, "cs123");
                        if (selectDraftFlag && PaymentBillPluginHelper.needSelectedDraft(recBill)) {
                            FormShowParameter showParameter = new FormShowParameter();
                            showParameter.setAppId("cas");
                            showParameter.setFormId("cas_receiver_selectdraft");
                            showParameter.setCustomParam("recBillId", selectedRowPks[0]);
                            showParameter.getOpenStyle().setShowType(ShowType.Modal);
                            showParameter.setCloseCallBack(new CloseCallBack(this, "CALLBACK_SELECTDRAFT"));
                            this.getView().showForm(showParameter);
                            args.setCancel(true);
                        }
                    }
                }
            }
        }

    }

    private Boolean checkStatus(Long id, String field) {
        List<QFilter> qFilters = new ArrayList();
        qFilters.add(new QFilter("id", "=", id));
        qFilters.add(new QFilter(field, "=", BillStatusEnum.AUDIT.getValue()));
        return QueryServiceHelper.exists("nckd_cas_recbill_xz", (QFilter[])qFilters.toArray(new QFilter[0]));
    }

    public void afterDoOperation(AfterDoOperationEventArgs e) {
        super.afterDoOperation(e);
        BillList list = (BillList)this.getView().getControl("billlistap");
        ListSelectedRowCollection selectedRows = list.getSelectedRows();
        boolean isSuccess = e.getOperationResult() == null ? false : e.getOperationResult().isSuccess();
        if ("cancelrefund".equals(e.getOperateKey()) && this.refundLogExecutor != null) {
            this.refundLogExecutor.saveLog(e.getOperationResult());
        }

        if ("receivingrec".equals(e.getOperateKey()) && isSuccess) {
            this.showRecTip();
            this.getView().invokeOperation("refresh");
        } else if (("cancelrec".equals(e.getOperateKey()) || "cancelrefund".equals(e.getOperateKey())) && isSuccess) {
            this.getView().invokeOperation("refresh");
        } else if ("transcompany".equals(e.getOperateKey()) && isSuccess) {
            this.getView().showSuccessNotification(ResManager.loadKDString("指定实际公司成功。", "ReceivingBillList_19", "fi-cas-formplugin", new Object[0]));
            this.getView().invokeOperation("refresh");
        } else if ("refund".equals(e.getOperateKey()) && isSuccess) {
            this.refund("refund");
        } else if ("returnmoney".equals(e.getOperateKey()) && isSuccess) {
            this.refund("returnmoney");
        } else if ("supplecontract".equals(e.getOperateKey()) && isSuccess) {
            this.suppleContractOrOrgsDivide("supplecontract");
        } else if ("orgsdivide".equals(e.getOperateKey()) && isSuccess) {
            this.suppleContractOrOrgsDivide("orgsdivide");
        } else if ("chargeback".equals(e.getOperateKey()) && isSuccess) {
            if (null != selectedRows && selectedRows.size() > 0) {
                CasPluginHelper.chargeBack(this, this.getView(), selectedRows.get(0).getPrimaryKeyValue(), "CALLBACK_RECCHARGEBACK");
            }
        } else if ("warnsubmit".equals(e.getOperateKey()) && isSuccess) {
            e.getOperationResult().setShowMessage(false);
        } else if ("matchdetail".equals(e.getOperateKey())) {
            ManualMatchHelper.bizBillMatchDetail(this.getView(), e, selectedRows, this, list.getEntityId(), "org,accountbank,currency");
        } else if (("marknoneedmatch".equals(e.getOperateKey()) || "cancelmark".equals(e.getOperateKey())) && isSuccess) {
            this.getView().invokeOperation("refresh");
        }

    }

    private void showRecTip() {
        BillList list = (BillList)this.getView().getControl("billlistap");
        ListSelectedRowCollection selectedRows = list.getSelectedRows();
        if (selectedRows.size() > 0) {
            List<Long> paymentBillPks = new LinkedList();
            Iterator var4 = selectedRows.iterator();

            while(var4.hasNext()) {
                ListSelectedRow selectedRow = (ListSelectedRow)var4.next();
                paymentBillPks.add((Long)selectedRow.getPrimaryKeyValue());
            }

            AppParam appParam = new AppParam();
            appParam.setAppId(TmcAppEnum.CAS.getId());
            QFilter payPidFilter = new QFilter("id", "in", paymentBillPks);
            DynamicObject[] recObjs = BusinessDataServiceHelper.load("nckd_cas_recbill_xz", "id,payer,billno,payertype,bizdate", new QFilter[]{payPidFilter});
            List<Long> orgIds = (List)Arrays.stream(recObjs).map((o) -> {
                return o.getLong("payer");
            }).collect(Collectors.toList());
            Set<Long> payerIsSupplierIds = this.getNeedQueryPayerId(recObjs, AsstActTypeEnum.SUPPLIER.getValue());
            Set<Long> payerIsCustomerIds = this.getNeedQueryPayerId(recObjs, AsstActTypeEnum.CUSTOMER.getValue());
            Map<Long, Long> supplierMap = CasPluginHelper.getInternalCompanyMap(AsstActTypeEnum.SUPPLIER.getValue(), payerIsSupplierIds);
            Map<Long, Long> customerMap = CasPluginHelper.getInternalCompanyMap(AsstActTypeEnum.CUSTOMER.getValue(), payerIsCustomerIds);
            Set<Long> orgIdSet = new HashSet(10);
            Iterator var13 = supplierMap.entrySet().iterator();

            Map.Entry entry;
            while(var13.hasNext()) {
                entry = (Map.Entry)var13.next();
                orgIdSet.add((Long) entry.getValue());
            }

            var13 = customerMap.entrySet().iterator();

            while(var13.hasNext()) {
                entry = (Map.Entry)var13.next();
                orgIdSet.add((Long) entry.getValue());
            }

            orgIdSet.addAll(orgIds);
            Map<String, Map<String, Object>> stringMapMap = SystemParamServiceHelper.loadBatchAppParameterByOrgFromCache(appParam, new ArrayList(orgIdSet));
            StringBuilder sb = new StringBuilder();
            DynamicObject[] var15 = recObjs;
            int var16 = recObjs.length;

            for(int var17 = 0; var17 < var16; ++var17) {
                DynamicObject recBill = var15[var17];
                Long payer = recBill.getLong("payer");
                String payerType = recBill.getString("payertype");
                Long internalCompany;
                if (AsstActTypeEnum.SUPPLIER.getValue().equals(payerType)) {
                    internalCompany = (Long)supplierMap.get(payer);
                    if (CasHelper.isEmpty(internalCompany)) {
                        continue;
                    }

                    payer = internalCompany;
                } else if (AsstActTypeEnum.CUSTOMER.getValue().equals(payerType)) {
                    internalCompany = (Long)customerMap.get(payer);
                    if (CasHelper.isEmpty(internalCompany)) {
                        continue;
                    }

                    payer = internalCompany;
                } else if (!AsstActTypeEnum.COMPANY.getValue().equals(payerType)) {
                    continue;
                }

                if (EmptyUtil.isNoEmpty(payer)) {
                    Date bizDate = recBill.getDate("bizdate");
                    DynamicObject currentPeriod = SystemStatusCtrolHelper.getCurrentPeriod(payer);
                    if (EmptyUtil.isNoEmpty(bizDate) && EmptyUtil.isNoEmpty(currentPeriod) && !bizDate.before(currentPeriod.getDate("begindate"))) {
                        continue;
                    }
                }

                Map<String, Object> sysParams = (Map)stringMapMap.get(String.valueOf(payer));
                if (EmptyUtil.isNoEmpty(sysParams)) {
                    Object CS048 = sysParams.get("cs048");
                    Object allowObj = sysParams.get("cs103");
                    if (EmptyUtil.isNoEmpty(CS048)) {
                        boolean isCheck = (Boolean)CS048;
                        if (isCheck && EmptyUtil.isNoEmpty(allowObj)) {
                            boolean allow = (Boolean)allowObj;
                            if (!allow) {
                                String billNo = recBill.getString("billno");
                                sb.append(String.format(ResManager.loadKDString("单据【%s】确认收款成功。因付款方设置不允许新增已结账期间的付款业务，因此收付协同生成付款单失败。\r\n", "ReceivingBillList_36", "fi-cas-formplugin", new Object[0]), billNo));
                            }
                        }
                    }
                }
            }

            if (EmptyUtil.isNotEmpty(sb)) {
                this.getView().showTipNotification(sb.toString());
            } else {
                this.getView().showSuccessNotification(ResManager.loadKDString("确认收款成功。", "ReceivingBillEdit_38", "fi-cas-formplugin", new Object[0]));
            }
        }

    }

    private Set<Long> getNeedQueryPayerId(DynamicObject[] payobjs, String type) {
        new HashSet(2);
        Set<Long> payeeIds = (Set)Arrays.stream(payobjs).map((o) -> {
            String payeeType = o.getString("payertype");
            return type.equals(payeeType) ? o.getLong("payer") : 0L;
        }).collect(Collectors.toSet());
        return payeeIds;
    }

    private void refund(String operateKey) {
        BillList billList = (BillList)this.getControl("billlistap");
        ListSelectedRowCollection selectedRows = billList.getSelectedRows();
        ListSelectedRow selectedRow = selectedRows.get(0);
        DynamicObject revBill = BusinessDataServiceHelper.loadSingleFromCache(selectedRow.getPrimaryKeyValue(), "nckd_cas_recbill_xz", "receivingtype.biztype");
        FormShowParameter parameter = new FormShowParameter();
        parameter.getOpenStyle().setShowType(ShowType.Modal);
        parameter.setCustomParam("billId", selectedRow.getPrimaryKeyValue());
        if (PayAndRecBizTypeEnum.SETTLCENTER_COLLECT.getValue().equals(revBill.getString("receivingtype.biztype"))) {
            parameter.setFormId("cas_refund");
            parameter.setCustomParam("operateKey", "returnmoney".equals(operateKey) ? "refund" : "returnmoney");
        } else {
            parameter.setFormId("cas_refund_renote");
            parameter.setCustomParam("operateKey", "returnmoney".equals(operateKey) ? "refund" : "renote");
        }

        CloseCallBack closeCallBack = new CloseCallBack(this, "refund");
        parameter.setCloseCallBack(closeCallBack);
        this.getView().showForm(parameter);
    }

    private void cancelrec() {
        BillList billList = (BillList)this.getControl("billlistap");
        ListSelectedRowCollection selectedRows = billList.getSelectedRows();
        if (selectedRows.size() <= 0) {
            this.getView().showTipNotification(ResManager.loadKDString("请选择要执行的数据。", "ReceivingBillList_21", "fi-cas-formplugin", new Object[0]));
        } else {
            List<Long> recBillPks = new LinkedList();
            Iterator var4 = selectedRows.iterator();

            while(var4.hasNext()) {
                ListSelectedRow selectedRow = (ListSelectedRow)var4.next();
                recBillPks.add((Long)selectedRow.getPrimaryKeyValue());
            }

            QFilter recPidFilter = new QFilter("id", "in", recBillPks);
            DynamicObjectCollection recobjs = QueryServiceHelper.query("nckd_cas_recbill_xz", "id,billno,isvoucher,org.id,receivingtype,receivingtype.enable", new QFilter[]{recPidFilter});
            StringBuilder sb = new StringBuilder();
            int flag = 0;
            Map<Long, List<DynamicObject>> groupOrg = (Map)recobjs.stream().collect(Collectors.groupingBy((o) -> {
                return o.getLong("org.id");
            }));
            Set<Long> orgIds = (Set)recobjs.stream().map((o) -> {
                return o.getLong("org.id");
            }).collect(Collectors.toSet());
            Map<String, Object> orgSysMap = SystemParameterHelper.getSystemParamsByOrgIds(new ArrayList(orgIds), "cs116");
            Iterator var11 = groupOrg.entrySet().iterator();

            while(true) {
                Map idBillNoMap;
                List existList;
                int size;
                do {
                    Map.Entry entry;
                    int CS114;
                    do {
                        if (!var11.hasNext()) {
                            if (flag == 1) {
                                this.getView().showConfirm(sb.substring(0, sb.length() - 1) + "，" + ResManager.loadKDString("是否继续取消收款？", "ReceivingBillList_23", "fi-cas-formplugin", new Object[0]), MessageBoxOptions.YesNo, ConfirmTypes.Default, new ConfirmCallBackListener("CONFIRM_CANCELREC_CALLBACK", this));
                                return;
                            } else {
                                if (flag > 0) {
                                    this.getView().showConfirm(String.format(ResManager.loadKDString("存在%d个已生成凭证的单据，是否继续取消收款？", "ReceivingBillList_24", "fi-cas-formplugin", new Object[0]), flag), sb.toString(), MessageBoxOptions.YesNo, ConfirmTypes.Default, new ConfirmCallBackListener("CONFIRM_CANCELREC_CALLBACK", this));
                                } else if (!CasPluginHelper.cancelCheckSettleForRec(this, this.getView(), recBillPks, recobjs)) {
                                    this.getView().invokeOperation("cancelrec");
                                    return;
                                }

                                return;
                            }
                        }

                        entry = (Map.Entry)var11.next();
                        Long orgId = (Long)entry.getKey();
                        CS114 = EmptyUtil.isEmpty(orgSysMap.get(String.valueOf(orgId))) ? 0 : Integer.parseInt((String)orgSysMap.get(String.valueOf(orgId)));
                    } while(3 != CS114);

                    List<DynamicObject> value = (List)entry.getValue();
                    idBillNoMap = (Map)value.stream().collect(Collectors.toMap((s) -> {
                        return s.getLong("id");
                    }, (v) -> {
                        return v.getString("billno");
                    }));
                    existList = VoucherBookHelper.getVoucherExistIds(idBillNoMap.keySet(), "nckd_cas_recbill_xz");
                    size = existList.size();
                    flag = size;
                } while(size <= 0);

                Iterator var19 = existList.iterator();

                while(var19.hasNext()) {
                    Long id = (Long)var19.next();
                    String billno = (String)idBillNoMap.get(id);
                    sb.append(String.format(ResManager.loadKDString("(%s)单据已经生成凭证\r\n", "ReceivingBillList_22", "fi-cas-formplugin", new Object[0]), billno));
                }
            }
        }
    }

    public void confirmCallBack(MessageBoxClosedEvent messageBoxClosedEvent) {
        super.confirmCallBack(messageBoxClosedEvent);
        String id = messageBoxClosedEvent.getCallBackId();
        MessageBoxResult res = messageBoxClosedEvent.getResult();
        if (StringUtils.equals(id, "CONFIRM_CANCELREC_CALLBACK") && MessageBoxResult.Yes.equals(res)) {
            BillList billList = (BillList)this.getControl("billlistap");
            ListSelectedRowCollection selectedRows = billList.getSelectedRows();
            if (selectedRows.size() > 0) {
                List<Long> recBillPks = new LinkedList();
                Iterator var7 = selectedRows.iterator();

                while(var7.hasNext()) {
                    ListSelectedRow selectedRow = (ListSelectedRow)var7.next();
                    recBillPks.add((Long)selectedRow.getPrimaryKeyValue());
                }

                QFilter recPidFilter = new QFilter("id", "in", recBillPks);
                DynamicObjectCollection recobjs = QueryServiceHelper.query("nckd_cas_recbill_xz", "id,billno,isvoucher", new QFilter[]{recPidFilter});
                if (!CasPluginHelper.cancelCheckSettleForRec(this, this.getView(), recBillPks, recobjs)) {
                    this.getView().invokeOperation("cancelrec");
                }
            }
        } else if (StringUtils.equals(id, "CONFIRM_CANCELPAY_SETTLE_CALLBACK") && MessageBoxResult.Yes.equals(res)) {
            this.getView().invokeOperation("cancelrec");
        } else if (id.equals("CONFIRM_DEL_CALLBACK")) {
            OperateOption option = OperateOption.create();
            if (MessageBoxResult.Yes.equals(res)) {
                option.setVariableValue("delrecpay", "1");
                this.getView().invokeOperation("delete", option);
            }
        } else if (StringUtils.equals(id, "CONFIRM_DELETE_CALLBACK")) {
            if (MessageBoxResult.Yes.equals(res)) {
                this.getView().invokeOperation("delete");
            }
        } else if ("CONFIRM_MANUALMATCH".equals(id) && MessageBoxResult.Yes.equals(res)) {
            ManualMatchHelper.confirmBizBillMatchDetail(this.getView(), messageBoxClosedEvent, "org,accountbank,currency");
        }

    }

    public void beforeItemClick(BeforeItemClickEvent evt) {
        String opKey = evt.getOperationKey();
        BillList billList = (BillList)this.getControl("billlistap");
        ListSelectedRowCollection selectRows = billList.getSelectedRows();
        if ("delete".equals(opKey) && selectRows.size() > 0) {
            Set<Object> selectRowsPks = new HashSet();
            selectRows.forEach((listSelectedRow) -> {
                selectRowsPks.add(listSelectedRow.getPrimaryKeyValue());
            });
            QFilter pkFilter = (new QFilter("id", "in", selectRowsPks)).and(new QFilter("hotaccount", "!=", "2"));
            QFilter sourceBillTypeFilter = new QFilter("sourcebilltype", "=", "cas_paybill");
            DynamicObject[] payBills = BusinessDataServiceHelper.load("nckd_cas_recbill_xz", "billno", new QFilter[]{pkFilter, sourceBillTypeFilter});
            if (payBills != null && payBills.length > 0) {
                evt.setCancel(true);
                StringBuilder billnos = new StringBuilder();
                Arrays.asList(payBills).forEach((payBill) -> {
                    billnos.append(payBill.getString("billno")).append("、");
                });
                this.getView().showConfirm(String.format(ResManager.loadKDString("%s收款单由内部收付协同业务自动生成，是否确认删除？", "ReceivingBillEdit_19", "fi-cas-formplugin", new Object[0]), billnos.substring(0, billnos.length() - 1)), MessageBoxOptions.YesNo, ConfirmTypes.Default, new ConfirmCallBackListener("CONFIRM_DEL_CALLBACK", this));
            }

            Object[] selectPks = selectRows.getPrimaryKeyValues();
            DynamicObject[] recSelectObj = BusinessDataServiceHelper.load(selectPks, MetadataServiceHelper.getDataEntityType("nckd_cas_recbill_xz"));
            List<DynamicObject> virtRecList = (List)Arrays.stream(recSelectObj).filter((o) -> {
                return o.getBoolean("isvirtual") && !EmptyUtil.isEmpty(o.getString("sourcebilltype")) && "nckd_cas_recbill_xz".equals(o.getString("sourcebilltype"));
            }).collect(Collectors.toList());
            if (virtRecList.size() > 0) {
                evt.setCancel(true);
                StringBuilder virtBillnos = new StringBuilder();
                virtRecList.forEach((rec) -> {
                    virtBillnos.append(rec.getString("billno")).append("、");
                });
                this.getView().showConfirm(String.format(ResManager.loadKDString("%s虚拟收款单由组织间清分业务触发自动下推的单据，请确认是否删除？", "ReceivingBillEdit_33", "fi-cas-formplugin", new Object[0]), virtBillnos.substring(0, virtBillnos.length() - 1)), MessageBoxOptions.YesNo, ConfirmTypes.Default, new ConfirmCallBackListener("CONFIRM_DELETE_CALLBACK", this));
            }
        }

    }

    public void setFilter(SetFilterEvent e) {
        super.setFilter(e);
        e.setOrderBy("bizdate desc,org.id desc,id desc");
    }
}
